<!-- BEGIN HWCC CONTEXT (auto-generated, do not edit) -->
{% if hot_context %}
{{ hot_context }}
{% else %}
# Hardware Context{% if project_name %} — {{ project_name }}{% endif %}

{% if mcu %}
- **MCU**: {{ mcu }}{% if architecture %} ({{ architecture }}{% if clock_mhz %}, {{ clock_mhz }}MHz{% endif %}){% endif %}

{% if flash_kb %}- **Flash**: {{ flash_kb }}KB
{% endif %}
{% if ram_kb %}- **RAM**: {{ ram_kb }}KB
{% endif %}
{% endif %}
{% endif %}
{% if register_access or error_handling or naming %}
## Coding Conventions
{% if register_access %}- Register access: {{ register_access }}
{% endif %}
{% if error_handling %}- Error handling: {{ error_handling }}
{% endif %}
{% if naming %}- Naming: {{ naming }}
{% endif %}
{% endif %}
Peripheral register maps are in `.rag/context/peripherals/`. Refer to these when writing hardware driver code.

> Generated by hwcc v{{ hwcc_version }}{% if generated_at %} on {{ generated_at }}{% endif %}

<!-- END HWCC CONTEXT -->
